@@ -362,6 +362,8 @@ class Hardware(BaseHardware):
     self.antenna_index = btn.index
     self.ChangeBandFilters()
   def ChangeBandFilters(self):
+    if not hasattr(self.application, "multi_rx_screen"):
+      return	# Needed for the VNA program
     self.SetControlBit(0x00, 13, self.antenna_index)
     highest = self.band
     freq = self.conf.BandEdge.get(highest, (0, 0))[0]
